<?php

namespace Drupal\views_cumulative_field\Plugin\views\field;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\PluralTranslatableMarkup;
use Drupal\views\Plugin\views\field\NumericField;
use Drupal\views\ResultRow;

/**
 * @file
 * Defines Drupal\views_cumulative_field\Plugin\views\field\CumulativeField.
 */

/**
 * Field handler to flag the node type.
 *
 * @ingroup views_field_handlers
 * @ViewsField("field_cumulative_field")
 */
class CumulativeField extends NumericField {

  /**
   * Sets the initial Cumulative Field data at zero.
   */
  public function query() {
    $this->additional_fields['cumulative_field_data'] = 0;
  }

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    $options['data_field'] = array('default' => NULL);
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);
    $form['data_field'] = array(
      '#type' => 'radios',
      '#title' => t('Data Field'),
      '#options' => $this->displayHandler->getFieldLabels(),
      '#default_value' => $this->options['data_field'],
      '#description' => t('Select the field for which to calculate the cumulative value.'),
      '#weight' => -10,
    );
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function getValue(ResultRow $values, $field = NULL) {
    parent::getValue($values, $field);
    $fieldName = $this->options['data_field'];
    $entity = $this->getEntity($values);
    $this->additional_fields['cumulative_field_data'] = $entity->get($fieldName)->getValue()[0]['value'] + $this->additional_fields['cumulative_field_data'];
    $value = $this->additional_fields['cumulative_field_data'];
    return $value;
  }

  /**
   * {@inheritdoc}
   */
  public function render(ResultRow $values) {
    $fieldName = $this->options['data_field'];
    $entity = $this->getEntity($values);
    $this->additional_fields['cumulative_field_data'] = $entity->get($fieldName)->getValue()[0]['value'] + $this->additional_fields['cumulative_field_data'];
    $value = $this->additional_fields['cumulative_field_data'];
    if (!empty($this->options['set_precision'])) {
      $value = number_format($value, $this->options['precision'], $this->options['decimal'], $this->options['separator']);
    }
    else {
      $remainder = abs($value) - intval(abs($value));
      $value = $value > 0 ? floor($value) : ceil($value);
      $value = number_format($value, 0, '', $this->options['separator']);
      if ($remainder) {
        // The substr may not be locale safe.
        $value .= $this->options['decimal'] . substr($remainder, 2);
      }
    }

    // Check to see if hiding should happen before adding prefix and suffix.
    if ($this->options['hide_empty'] && empty($value) && ($value !== 0 || $this->options['empty_zero'])) {
      return '';
    }

    // If we should format as plural, take the (possibly) translated plural
    // setting and format with the current language.
    if (!empty($this->options['format_plural'])) {
      $value = PluralTranslatableMarkup::createFromTranslatedString($value, $this->options['format_plural_string']);
    }

    return $this->sanitizeValue($this->options['prefix'], 'xss') . $this->sanitizeValue($value) . $this->sanitizeValue($this->options['suffix'], 'xss');
  }

}
